@@ -390,15 +390,31 @@ class UTestCase(unittest.TestCase):
 
 
     def assertLinf(self, first, second=None, tol=1e-5, msg=None):
+        """ Test in the L_infinity norm.
+        :param first:
+        :param second:
+        :param tol:
+        :param msg:
+        :return:
+        """
         if second is None:
             return self.wrap_assert(self.assertLinf, first, tol=tol, msg=msg)
         else:
             diff = self._shape_equal(first, second)
-            if max(diff.flat) >= tol:
+            np.testing.assert_allclose(first, second, atol=tol)
+            
+            max_diff = max(diff.flat)
+            if max_diff >= tol:
                 from unittest.util import safe_repr
                 # msg = f'{safe_repr(first)} != {safe_repr(second)} : Not equal within tolerance {tol}'
                 # print(msg)
-                self.assertEqual(first, second, msg=f'Not equal within tolerance {tol}')
+                # np.testing.assert_almost_equal
+                # import numpy as np
+                print(f"|first - second|_max = {max_diff} > {tol} ")
+                np.testing.assert_almost_equal(first, second, msg=f'Not equal within tolerance {tol}')
+                # If the above fail, make sure to throw an error:
+                self.assertFalse(max_diff >= tol, msg=f'Input arrays are not equal within tolerance {tol}')
+                # self.assertEqual(first, second, msg=f'Not equal within tolerance {tol}')
 
     def assertL2(self, first, second=None, tol=1e-5, msg=None, relative=False):
         if second is None:
@@ -412,10 +428,15 @@ class UTestCase(unittest.TestCase):
             diff = ( ( np.asarray( diff.flatten() )**2).sum() )**.5
 
             scale = (2/(np.linalg.norm(np.asarray(first).flat) + np.linalg.norm(np.asarray(second).flat)) ) if relative else 1
-            if max(diff.flat)*scale >= tol:
+            max_diff = diff*scale
+            if max_diff >= tol:
                 msg = "" if msg is None else msg
-                print(f"|first - second|_2 = {diff} > {tol} ")
-                self.assertEqual(first, second, msg=msg + f"Not equal within tolerance {tol}")
+                print(f"|first - second|_2 = {max_diff} > {tol} ")
+                # Deletage to numpy. Let numpy make nicer messages.
+                np.testing.assert_almost_equal(first, second, msg=f'Not equal within tolerance {tol}')
+                # Make sure to throw an error no matter what.
+                self.assertFalse(max_diff >= tol, msg=f'Input arrays are not equal within tolerance {tol}')
+                # self.assertEqual(first, second, msg=msg + f"Not equal within tolerance {tol}")
 
     def _cache_file(self):
         return os.path.dirname(inspect.getabsfile(type(self))) + "/unitgrade_data/" + self.__class__.__name__ + ".pkl"
